<?php
/*
Plugin Name: txt.me
Plugin URI: https://txt.me/
Description: Add txt.me Live Chat Widget to your site
Version: 1.0.2
Author: txtme
Author URI: https://profiles.wordpress.org/txtme/
*/

/*
     Copyright 2021  txt.me  (email : support {at} txt.me)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

class txtme {
    public function __construct()
    {
        add_option('txtme_login');
        add_option('txtme_password');
        add_option('txtme_is_authenticated', 'false');
        add_option('txtme_userSlug');
        add_option('txtme_apiKey');
        add_option('txtme_websiteSlug', 'websiteSlug');
        add_option('txtme_nonceAction');
        add_option('txtme_nonceValue');

        add_action('admin_menu', array($this, 'admin'));
        add_action('wp_enqueue_scripts', array($this, 'my_scripts_method'), 99, 1);
        add_filter('script_loader_tag', array($this, 'namespace_async_scripts'), 10, 2);
    }

    public function namespace_async_scripts($tag, $handle)
    {
        // Just return the tag normally if this isn't one we want to async
        if ('txt-me' !== $handle) {
            return $tag;
        }

        return str_replace(' src', ' async src', $tag);
    }

    public function my_scripts_method()
    {
        $meta = get_option('txtme_websiteSlug');
        if (empty($meta)) {
            return;
        }
        if (trim($meta) === '') {
            return;
        }
        wp_footer();
        wp_register_script('txt-me', 'https://v2.txt.me/livechat/js/wrapper/' . $meta, array(), false, true);
        wp_enqueue_script('txt-me');
    }


    public function admin()
    {
        if (function_exists('add_menu_page')) {
            add_menu_page(
                'txt.me Options',
                'txt.me',
                'administrator',
                basename(__FILE__),
                array(&$this, 'admin_form'),
                plugins_url('img/logo_small.png?1.0.1', __FILE__)
            );
        }
    }

    public function admin_form() {
        $txtme_websiteSlug = get_option( 'txtme_websiteSlug' );
        $nonceAction       = get_option( 'txtme_nonceAction' );
        $nonceValue        = get_option( 'txtme_nonceValue' );

        if (function_exists('wp_create_nonce')) {
            if (isset($_GET['_wpnonce'])) {
                if (wp_verify_nonce($_GET['_wpnonce'], $nonceAction)) {
                    if (function_exists('current_user_can') && !current_user_can('manage_options')) {
                        die ('Hacker?');
                    }
                    $txtme_userSlug = sanitize_text_field($_GET['userId']);
                    update_option('txtme_userSlug', $txtme_userSlug);
                    $txtme_apiKey = sanitize_text_field($_GET['token']);
                    update_option('txtme_apiKey', $txtme_apiKey);

                    update_option('txtme_is_authenticated', 'true');
                    ?>
                    <div class="notice notice-success is-dismissible">
                        <p><?php echo translate('Successfully logged in!', 'txtme'); ?></p>
                    </div>
                    <?php
                } else {
                    ?>
                    <div class="notice notice-error is-dismissible">
                        <p><?php echo translate('Wrong nonce. Possible request was forged', 'txt-me'); ?></p>
                    </div>
                    <?php
                }
            } else {
                $nonceAction = wp_generate_password('21', false);
                update_option('txtme_nonceAction', $nonceAction);
                $nonceValue = wp_create_nonce($nonceAction);
                update_option('txtme_nonceValue', $nonceValue);
            }
        } else {
            ?>
            <div class="notice notice-error is-dismissible">
                <p><?php echo translate('Your WP is too old please upgrade it.', 'txt-me'); ?></p>
            </div>
            <?php
        }

        if (isset($_POST['logout'])) {
            if (function_exists('current_user_can') && !current_user_can('manage_options')) {
                die ('Hacker?');
            }

            if (function_exists('check_admin_referer')) {
                check_admin_referer('txtme_logout_form');
            }

            update_option('txtme_is_authenticated', 'false');
            update_option('txtme_userSlug', '');
            update_option('txtme_apiKey', '');
            update_option('txtme_websiteSlug', '');
//			update_option( 'txtme_nonceValue', '' );
//			update_option( 'txtme_nonceAction', '' );
            ?>
            <div class="notice notice-success is-dismissible">
                <p><?php echo translate('Successfully log out from txt.me', 'txtme'); ?></p>
            </div>
            <?php
            update_option('txtme_login', '');
            update_option('txtme_password', '');
        }

        if (isset($_POST['select'])) {
            if (function_exists('current_user_can') && !current_user_can('manage_options')) {
                die ('Hacker?');
            }

            if (function_exists('check_admin_referer')) {
                check_admin_referer('txtme_select_form');
            }

            if (isset($_POST['websiteSlug'])) {
                if ($txtme_websiteSlug === 'websiteSlug') {
                    ?>
                    <div class="notice notice-success is-dismissible">
                        <p><?php echo translate('Widget successfully selected for txt.me', 'txt-me'); ?></p>
                    </div>
                    <?php
                    update_option('txtme_websiteSlug', sanitize_text_field($_POST['websiteSlug']));
                } elseif ($txtme_websiteSlug !== $_POST['websiteSlug']) {
                    ?>
                    <div class="notice notice-success is-dismissible">
                        <p><?php echo translate('Widget successfully updated for txt.me', 'txt-me'); ?></p>
                    </div>
                    <?php
                    update_option('txtme_websiteSlug', sanitize_text_field($_POST['websiteSlug']));
                }
            }
        }

        $txtme_is_authenticated = get_option('txtme_is_authenticated');
        $txtme_userSlug = get_option('txtme_userSlug');
        $txtme_apiKey = get_option('txtme_apiKey');
        $txtme_websiteSlug = get_option('txtme_websiteSlug');

        $login_url = ('https://txt.me/admin/login?callbackUrl=' . urlencode(network_admin_url('admin.php', __FILE__) . '?page=txtme.php&_wpnonce=' . $nonceValue));
        ?>

        <?php if ($txtme_is_authenticated === 'true') { ?>
            <div class="wrap">
                <h2><img src="<?php echo plugins_url('img/logo.png?1.0.1', __FILE__); ?>" alt=""/></h2>
                <h3><?php echo translate("You logged in ", "txtme"); ?></h3>
                <form name="logout" method="post" action="<?php echo esc_url(network_admin_url('admin.php', __FILE__)) . '?page=txtme.php'; ?>">
                    <?php
                    if (function_exists('wp_nonce_field')) {
                        wp_nonce_field('txtme_logout_form');
                    }
                    ?>
                    <input type="hidden" name="action" value="update"/>
                    <input type="hidden" name="page_options" value="txtme_login,txtme_password,txtme_is_authenticated,txtme_userSlug,txtme_apiKey"/>
                    <p class="submit">
                        <input type="submit" name="logout" value="<?php echo translate('Logout', 'txtme'); ?>"/>
                    </p>
                </form>
            </div>

            <?php
            $response = wp_remote_get('https://v2.txt.me/livechat/v1/api/users/' . $txtme_userSlug . '/websites', array(
                'timeout' => 5,
                'redirection' => 5,
                'httpversion' => '1.0',
                'blocking' => true,
                'headers' => array("Content-Type" => "application/json", "x-api-key" => $txtme_apiKey),
                'body' => array(),
                'cookies' => array()
            ));

            if (is_wp_error($response)) {
                $error_message = $response->get_error_message();
                echo "Something goes wrong: $error_message";
            } elseif (isset($response['body'])) {
                $bodyObject = json_decode($response['body'], true);
                if (count($bodyObject) >= 1) {
                    ?>
                    <div class="wrap">
                        <h2><?php echo translate('txt.me List of Widgets', 'txtme'); ?></h2>
                        <form name="select" method="post" action="<?php echo esc_url(network_admin_url( 'admin.php', __FILE__ )) . '?page=txtme.php'; ?>">
                            <?php
                            if (function_exists('wp_nonce_field')) {
                                wp_nonce_field('txtme_select_form');
                            }
                            ?>

                            <table class="form-table">
                                <tr>
                                    <th scope="row">Not selected</th>
                                    <td>
                                        <input type="radio" id="contactChoice0" name="websiteSlug" value=""<?php echo (empty($txtme_websiteSlug) ? ' checked' : ''); ?>/>
                                    </td>
                                </tr>
                                <?php
                                foreach ($bodyObject as $item) {
                                    echo '    <tr>';
                                    echo '        <th scope="row">' . $item["name"] . '</th>';
                                    echo '        <td>';
                                    echo '            <input type="radio" id="contactChoice' . $item["slug"] . '" name="websiteSlug" value="' . $item["slug"] . '"' . ($txtme_websiteSlug === $item["slug"] ? ' checked' : '') . '/>';
                                    echo '        </td>';
                                    echo '    </tr>';
                                }
                                ?>
                            </table>
                            <input type="hidden" name="action" value="update"/>
                            <input type="hidden" name="page_options" value="txtme_websiteSlug"/>
                            <p class="submit">
                                <input type="submit" name="select" value="<?php echo translate('Save selection', 'txtme'); ?>"/>
                            </p>
                        </form>
                    </div>
                    <?php
                }
            } else {
                echo "Something goes wrong: Response not have body part";
            }
        } else {
            ?>
            <div class="wrap">
                <img src="<?php echo plugins_url('img/logo.png?1.0.1', __FILE__); ?>" alt=""/>
                <h2><?php echo translate('txt.me Login Link', 'txtme'); ?></h2>
                <a href="<?php echo $login_url; ?>" target="_blank">Click to Login/Sign Up</a>
            </div>
            <?php
        }
    }
}
$txtme = new txtme();